unit ComThreads;

interface

uses
  Windows, SysUtils, Messages, Classes;

const
  AM_ERRORSTR = WM_USER + $99;
  AM_BYTECT = WM_USER + $98;
  LItsMe = $12DCCA5A;

type
  TComThread = class(TThread)
  public
    BufLen : LongWord;
    Once: Boolean;
    constructor Create(hNewOutCom, hNewInCom, hNewRaceEvent, hNewErrWnd: THandle; combufLen:Longword; Once:Boolean; bNewAlternate: Boolean = False); virtual;
    procedure SendErr(ErrStr: AnsiString); virtual;
  protected
    hOutCom, hInCom, hRaceEvent, hErrWnd: THandle;
    bAlternate: Boolean;

    procedure Execute; override;
  end;

  TWriteWorkerThread = class(TThread)
  public
    Once: Boolean;
    MyComThread: TComThread;
    hCom, hBufMutex, hBufSem: THandle;
    StrBuf: array of AnsiString;
  protected
    procedure Execute; override;
  end;

function PersistentReadFile(hFile: THandle; pBuf: PAnsiChar; BufLen: LongWord; var BytesRead: LongWord): LongWord;
function PersistentWriteFile(hFile: THandle; pBuf: PAnsiChar; BufLen: LongWord; var BytesWritten: LongWord): LongWord;


implementation

function StatusBE(bSucc: Boolean): LongWord;
begin
  if bSucc then
    Result := ERROR_SUCCESS
  else begin
    Result := GetLastError;
    if Result = ERROR_SUCCESS then Result := ERROR_GEN_FAILURE;
  end;
end;

function PersistentReadFile(hFile: THandle; pBuf: PAnsiChar; BufLen: LongWord; var BytesRead: LongWord): LongWord;
var
  ThisBytesRead: LongWord;

begin
  BytesRead := 0;
  repeat
    ThisBytesRead := 0;
    Result := StatusBE(ReadFile(hFile, pBuf^, BufLen, ThisBytesRead, nil));
    Inc(BytesRead, ThisBytesRead);
    if Result <> ERROR_SUCCESS then Break;
    if ThisBytesRead >= BufLen then Break;
    Dec(BufLen, ThisBytesRead);
    Inc(pBuf, ThisBytesRead);
  until False;
end;

function PersistentWriteFile(hFile: THandle; pBuf: PAnsiChar; BufLen: LongWord; var BytesWritten: LongWord): LongWord;
var
  ThisBytesWritten: LongWord;

begin
  BytesWritten := 0;
  repeat
    ThisBytesWritten := 0;
    Result := StatusBE(WriteFile(hFile, pBuf^, BufLen, ThisBytesWritten, nil));
    Inc(BytesWritten, ThisBytesWritten);
    if Result <> ERROR_SUCCESS then Break;
    if ThisBytesWritten >= BufLen then Break;
    Dec(BufLen, ThisBytesWritten);
    Inc(pBuf, ThisBytesWritten);
  until False;
end;

function MehRandByte: Byte;
var
  Roll: LongWord;

begin
  //This isn't thread-safe. We don't care.
  repeat
    Roll := Random(257);
  until Roll < $100;
  Result := ord('U');//Roll; //--J2H Debug, capital U for debugging on scope
end;

{ TComThread }

constructor TComThread.Create(hNewOutCom, hNewInCom, hNewRaceEvent, hNewErrWnd: THandle; combufLen:Longword;Once:Boolean; bNewAlternate: Boolean );
begin
  inherited Create(True);
  hOutCom := hNewOutCom;
  hInCom := hNewInCom;
  hRaceEvent := hNewRaceEvent;
  hErrWnd := hNewErrWnd;
  bAlternate := bNewAlternate;
  BufLen := combufLen;
  Resume;

end;

procedure TComThread.Execute;
var
  MyWriteWorker: TWriteWorkerThread;
  OComStr, IComStr: AnsiString;
  I, FirstMismatch, MBCt: Integer;
  Status, BytesRead, Mismatches: LongWord;

begin
  MBCt := 0;
  
  MyWriteWorker := TWriteWorkerThread.Create(True);
  MyWriteWorker.hBufMutex := CreateMutex(nil, False, nil);
  MyWriteWorker.hBufSem := CreateSemaphore(nil, 0, $FFFF, nil);
  MyWriteWorker.hCom := hOutCom;
  MyWriteWorker.Once := Once;
  MyWriteWorker.Resume;

  WaitForSingleObject(hRaceEvent, INFINITE);

  repeat
    SetString(OComStr, nil, BufLen);
    SetString(IComStr, nil, BufLen);
    for I := 1 to BufLen do OComStr[I] := AnsiChar(MehRandByte);
    WaitForSingleObject(MyWriteWorker.hBufMutex, INFINITE);
      I := Length(MyWriteWorker.StrBuf);
      SetLength(MyWriteWorker.StrBuf, I+1);
      MyWriteWorker.StrBuf[I] := OComStr;
    ReleaseMutex(MyWriteWorker.hBufMutex);
    ReleaseSemaphore(MyWriteWorker.hBufSem, 1, nil);

    Status := PersistentReadFile(hInCom, PAnsiChar(IComStr), BufLen, BytesRead);

    if Status <> ERROR_SUCCESS then begin
      SendErr('Error ' + IntToStr(Status) + ' from ReadFile.');
    end
    else begin
{
      Mismatches := 0;
      FirstMismatch := -1;
      for I := 1 to Length(IComStr) do
        if IComStr[I] <> OComStr[I] then begin
          if Mismatches = 0 then FirstMismatch := I;
          Inc(Mismatches);
        end
      ;
      if Mismatches <> 0 then begin
        SendErr(IntToStr(Mismatches) + ' mismatches. First(at [' + IntToStr(FirstMismatch-1) + ']) expected ' + IntToHex(Byte(OComStr[FirstMismatch]), 2) + ', got ' + IntToHex(Byte(IComStr[FirstMismatch]), 2) + '.');
      end
      else
}
      begin
        Inc(MBCt);
        PostMessage(hErrWnd, AM_BYTECT, MBCt, LItsMe);
      end;
    end;

    if bAlternate then begin
      MyWriteWorker.hCom := hInCom;
      hInCom := hOutCom;
      hOutCom := MyWriteWorker.hCom;
    end;
  until Once or Terminated;

  ReleaseSemaphore(MyWriteWorker.hBufSem, 1, nil); //Sem release without string is kill code.
  //It doesn't check the return from the waits, so this lets it happily fall through.
  CloseHandle(MyWriteWorker.hBufMutex);
  CloseHandle(MyWriteWorker.hBufSem);
  FreeAndNil(MyWriteWorker);      
end;

procedure TComThread.SendErr(ErrStr: AnsiString);
begin
  SendMessage(hErrWnd, AM_ERRORSTR, Integer(@(ErrStr)), LItsMe);
  ErrStr := '';
end;

{ TWriteWorkerThread }

procedure TWriteWorkerThread.Execute;
var
  bGotOne: Boolean;
  I: Integer;
  ComStr: AnsiString;
  Status, BytesWritten: LongWord;

begin
  repeat
    WaitForSingleObject(hBufSem, INFINITE);
    WaitForSingleObject(hBufMutex, INFINITE);
      I := High(StrBuf);
      bGotOne := I >= 0;
      if bGotOne then begin
        ComStr := StrBuf[0];
        if I > 0 then begin
          StrBuf[0] := StrBuf[I]; //For the garbage collector.
          Move(StrBuf[1], StrBuf[0], SizeOf(Pointer) * I);
        end;
        SetLength(StrBuf, I);
      end;
    ReleaseMutex(hBufMutex);
    if not bGotOne then Break; //Sem release without string is kill code.

    Status := PersistentWriteFile(hCom, PAnsiChar(ComStr), Length(ComStr), BytesWritten);
    if Status <> ERROR_SUCCESS then begin
      MyComThread.SendErr('Error ' + IntToStr(Status) + ' from WriteFile.');
    end
  until Once;// False; --J2H Use "False" for repeating, Once is "once if cbOnce is checked
end;

end.
