unit J2HUtils;
interface

uses Classes, Graphics;

function FirstNonNumber(S: AnsiString): Integer;
function TintColor(MainColor, TintColor: TColor; AmountOfTint: Single): TColor;
function FormattedStrToInt(s:String):Integer;
function FormattedStrToFloat(s:String):Double;
Function FormattedFloatToStr(v:double):string;
Function FormattedIntToStr(v:Integer):string;
procedure TokenizeCSV(S : string; Strings : TStringList; delim:Char=',');
function UnTokenizeCSV(S:TStringList):String;
function IntToBin(I, MinBits: LongWord): String;
function BinToNybbledBin(b:string):string;

implementation

uses SysUtils;



function FirstNonNumber(S: AnsiString): Integer;
begin
  for Result := 1 to Length(S) do
    if (S[Result] < '0') or (S[Result] > '9') then
      Exit
  ;
  Result := 0;
end;

function TintColor(MainColor, TintColor: TColor; AmountOfTint: Single): TColor;
  type
    TRGBFColor = packed record
      case Boolean of
      True: (
        Color: TColor;
      );
      False: (
        Red: Byte;
        Green: Byte;
        Blue: Byte;
        Flags: Byte;
      );
    end;

  var
    FC: TRGBFColor;
    BC: TRGBFColor;
    RC: TRGBFColor absolute Result;
    ANess: Single;

begin
  FC.Color := ColorToRGB(MainColor);
  BC.Color := ColorToRGB(TintColor);
  ANess := 1 - AmountOfTint;
  RC.Red   := Round(ANess * FC.Red   + AmountOfTint * BC.Red  );
  RC.Green := Round(ANess * FC.Green + AmountOfTint * BC.Green);
  RC.Blue  := Round(ANess * FC.Blue  + AmountOfTint * BC.Blue );
  RC.Flags := 0;
end;

function FormattedStrToInt(s:String):Integer;
  var
    iSrc, iDst: integer;
    d: string;
    foo: Integer;
begin
  if length(s) > 0 then
  begin
    SetLength(d, 255);
    iDst := 1;
    for iSrc := 1 to Length(s) do begin
      if s[iSrc] = ',' then continue;
      d[iDst] := s[iSrc];
      inc(iDst);
    end;
    SetLength(d,iDst-1);
    foo := 0;
    try
      foo := StrToInt(d);
    except end;
    Result := foo;
  end else begin
    Result := 0;
  end;
end;

function FormattedStrToFloat(s:String):Double;
  var
    foo: Double;
    iSrc, iDst: integer;
    d: string;
begin
  SetLength(d,255);
  iDst := 1;
  for iSrc := 1 to Length(s) do begin
    if s[iSrc] = ',' then continue;
    d[iDst] := s[iSrc];
    inc(iDst);
  end;
  SetLength(d,iDst-1);
  foo := 0.0;
  try
    foo := StrToFloat(d);
  except end;
  result := foo;
end;

Function FormattedFloatToStr(v:double):string;
begin
  Result := FloatToStrF(v, ffFixed, 7, 2); // change to insert friendly commas
end;

Function FormattedIntToStr(v:Integer):string;
begin
  Result := IntToStr(v); // change to insert friendly commas
end;

procedure TokenizeCSV(S : string; Strings : TStringList; delim:Char);
  var
    C : Integer;
    L : Integer;
    Field : string;
    Quote : boolean;
begin
  L := Length(S);
  Strings.Clear;
  if L > 0 then
  begin
    C := 1;
    repeat
      Field := '';
      if S[C] = '"' then
        begin
                Inc(C);
                repeat
                  while S[C] <> '"' do
                  begin
                    Field := Field + S[C];
                    Inc(C);
                  end;
                  Quote := False;
                  while (C < L) and (S[C] = '"') do
                  begin
                    Inc(C);
                    if Quote then Field := Field + '"';
                    Quote := not Quote;
                  end;
                until (C >= L) or Quote;
        end
        else
        begin
                while (C <= L) do
                begin
                    if (S[C] <> delim) then
                    begin
                      Field := Field + S[C];
                      Inc(C);
                    end else break;  
                end;
        end;
        Strings.Add(Field);
        Inc(C);
    until C >= L;
    Strings.Add('');
  end;
end;

function UnTokenizeCSV(S: TStringList): String;
begin
  Result := StringReplace( Trim(S.Text), #13#10, ', ', [rfReplaceAll]);
end;

function IntToBin(I, MinBits: LongWord): String;
  var
    DoneBits: LongWord;
begin
  Result := '';
  DoneBits := 0;
  while ((I <> 0) or (DoneBits < MinBits)) do
  begin
    Result := Char(Byte('0') + (I and 1)) + Result;
    I := I shr 1;
    Inc(DoneBits);
  end;
  if Result = '' then result := '0';
end;

function BinToNybbledBin(b:string):string;
  var
    nybble: integer;
begin
  for nybble := 0 to (length(b) div 4) do
    Result := Result + copy(b,nybble*4+1, 4) + ' ';
end;
end.
