unit MainUnit;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs, StdCtrls, ExtCtrls,
  ComThreads;

const
  AM_STOP = WM_USER + $89;

type
  TMainForm = class(TForm)
    Panel1: TPanel;
    GroupBox1: TGroupBox;
    COMEditA: TEdit;
    GroupBox2: TGroupBox;
    COMEditB: TEdit;
    BaudLabel: TLabel;
    ResultMemo: TMemo;
    COMLabel: TLabel;
    Panel2: TPanel;
    lbPossibleBauds: TListBox;
    lbWaysToGetBaud: TListBox;
    Panel3: TPanel;
    cbTestOnSelectBaud: TCheckBox;
    Label1: TLabel;
    Label2: TLabel;
    cbN: TComboBox;
    cbM: TComboBox;
    cbSCR: TComboBox;
    cbDLL: TComboBox;
    lbBaudCodeBinary: TMemo;
    Label3: TLabel;
    baudCodeUSR: TEdit;
    baudCodeDLH: TEdit;
    baudCodeDLL: TEdit;
    baudCodeM: TEdit;
    baudCodeN: TEdit;
    baudCodeSCR: TEdit;
    Label4: TLabel;
    Label5: TLabel;
    Label6: TLabel;
    Label7: TLabel;
    Label8: TLabel;
    Label9: TLabel;
    Label10: TLabel;
    Button4: TButton;
    Button5: TButton;
    Button2: TButton;
    Button1: TButton;
    Button3: TButton;
    Button6: TButton;
    Button7: TButton;
    Button8: TButton;
    procedure FormCreate(Sender: TObject);
    procedure COMEditChange(Sender: TObject);
    procedure lbPossibleBaudsClick(Sender: TObject);
    procedure lbWaysToGetBaudClick(Sender: TObject);
    procedure Button1Click(Sender: TObject);
  private
    procedure ListPossibleBaudCodes(desiredBaud: string = '');
    function IntToBaudCodeVerbose(v: Longword; verbose: Boolean=False): String;
  protected
    hComA, hComB, hRaceEvent: THandle;
    WorkerThread: array of TComThread;
    ErrCt: Integer;
    bAborted: Boolean;

    procedure DisableControls;
    procedure EnableControls;
    procedure AMErrorStr(var Msg: TMessage); message AM_ERRORSTR;
    procedure AMStop(var Msg: TMessage); message AM_STOP;
    procedure AMByteCt(var Msg: TMessage); message AM_BYTECT;
    { Private declarations }
  public
    { Public declarations }
  end;

var
  MainForm: TMainForm;

implementation
uses J2HUtils;
{$R *.DFM}

const
  SlowClock = 14745600;
  FastClock = 48000000;
  separator = #9;

{

--J2H documentation, from an email with Pericom, updated 20150505:

    We re-define the parameter of "Baudrate" on DCB structure,
    Bit[30:28]: User Define Baudrate, this value have to be non-zero
    Bit[27:24]: the value of SC [SampleClock]
    Bit[23:16]: the value of DLH
    Bit[15:8]: the value of DLL
    Bit[7:3]: the value of M, (0 <= M <= 2) // even though (M: 0..$1F) would fit ...
    Bit[2:0]: the value of N, (0 <= N <= 7)

    If you want to set
    DLL=1
    DLH=0
    SC=12
    then, the parameter of Baudrate on DCB structure should be set to 0x1c000108.
    When our driver find the parameter "Baudrate" does not exist on the default baudrate table,
    and Bit 28 is set to "1", it will get the values of SCR, DLL, DLH, M and N from Baudrate
    parameter and set to registers directly.
}

type TbaudInfo = record
  Rate: double;
  Code: Longword;
  N: 0..7;
  M: 0..2;
  DLL: byte;
  DLH: byte;
  SCR:0..$F;   // previously was $0..$B ???
  SC: 5..$F;
end;

function CalcBaudCode(DL, Prescaler: LongWord): TbaudInfo;
  var
    ThisPrescaler, N, M, SCR: LongWord;
begin
  for N := 0 to 7 do
    for M := 0 to 2 do
      for SCR := $0 to $B do
      begin
        ThisPrescaler := (1 shl M) * ((16 - SCR) + N);         // -- J2H: Corrected, "2^M", not "2^(M-1)"
        if ThisPrescaler = Prescaler then
        begin
          Result.Code := (N and $7) // DW fix, was (N and $F).
                             or ((M and $1F) shl 3)
                             or ((DL and $FFFF) shl 8)
                             or ((SCR and $F) shl 24)
                             or $70000000                         // --J2H Fixed?
          ;
          Result.N := N;
          Result.M := M;
          Result.SCR := SCR;
          Result.SC := SCR;
          Result.DLL := DL and $FF;
          Result.DLH := DL div 8;
          Exit;
        end;
      end
  ;
end;

procedure TMainForm.ListPossibleBaudCodes(desiredBaud: string='');
var
  D, N, M, SCR: Longword;
  Baud : double;
  sBaud, msg: AnsiString;
  index: integer;
  BaudInfo: TBaudInfo;

  function SampleClock: Longword;
  begin
    Result := 16 - SCR;
  end;

  function Prescaler:longword;
  begin
    Result := (1 shl M) * (SampleClock + N);
  end;

begin
  lbWaysToGetBaud.Sorted := True;
  lbWaysToGetBaud.Clear;
  lbPossibleBauds.Sorted := True;

  for D := 1 to 50 do
    for N := 0 to 7 do
      for M := 0 to 2 do
        for SCR := $0 to $B do begin
          Baud := FastClock / (D * Prescaler);
          sBaud := FloatToStrF(Baud, ffFixed, 9, 0);
          while( length(sBaud) < 8) do sBaud := ' ' + sBaud;

          if desiredBaud = '' then
            if lbPossibleBauds.items.IndexOf(sBaud) < 0 then
              lbPossibleBauds.Items.Add(sBaud)
          ;

          if (desiredBaud = '') or (desiredBaud = sBaud) then begin
            msg :=   sBaud + '= /'
                  + IntToStr(PreScaler) + separator
                  + IntToStr(N) + separator
                  + IntToStr(M) + separator
                  + IntToStr(SCR) + separator  //--J2H Debug sampleclock...
                  + IntToStr(D) + separator
                  + FloatToStrF(SlowClock / (D * Prescaler), ffFixed, 9, 0) + separator
                  + '  ('+ FloatToStrF(1000000000/(SlowClock / (D * Prescaler)), ffFixed, 9, 3) + 'ns)'
            ;
            index := lbWaysToGetBaud.items.Add(msg);
          end;
        end
  ;
  lbWaysToGetBaud.Sorted := False;
  
  msg:='/prescaler' + separator
        + 'N' + separator
        + 'M' + separator
        + 'SCR' + separator
        + 'D' + separator
        + 'baud ' + separator
        + 'bitlength'
        ;
  lbWaysToGetBaud.Items.Insert(0, msg);
end;


function CreateComFile(ComStr: AnsiString): THandle;
begin
  Result := CreateFile(PChar(ComStr), GENERIC_READ or GENERIC_WRITE, 0, nil, OPEN_EXISTING, 0, 0);
  ComStr := '';
end;

procedure ConfigureCom(hCom: THandle; BaudRate: LongWord);
var
  Tim: COMMTIMEOUTS;
  ComDCB: DCB;

begin
  SetupComm(hCom, 1026, 255); // In buffer = 1026 bytes, out buffer = 255 bytes

  Tim.ReadIntervalTimeout := 5;
  Tim.ReadTotalTimeoutConstant := 1000;    // No read total timeout, we want to get all the data that comes our way
  Tim.ReadTotalTimeoutMultiplier := 1;
  Tim.WriteTotalTimeoutConstant := 1000;   // No write timeout.
  Tim.WriteTotalTimeoutMultiplier := 1;
  SetCommTimeouts(hCom, Tim);

  GetCommState(hCom, ComDCB); // Fill ComDCB with current data
  ComDCB.BaudRate := BaudRate;
  ComDCB.ByteSize := 7;
  ComDCB.Parity   := EVENPARITY;
  ComDCB.StopBits := ONESTOPBIT;
  if SetCommState(hCom, ComDCB) = true then
    ShowMessage('error configuring ' + IntToStr(ComDCB.BaudRate) + ' (' + IntToHex(ComDCB.BaudRate,8) + ')')
  else begin
    GetCommState(hCom, ComDCB); // Fill ComDCB with current data
    ShowMessage('Baud set to ' + IntToStr(ComDCB.BaudRate) + ' (' + IntToHex(ComDCB.BaudRate,8) + ')');
  end;
end;

function C2alcTimestampPrefix: AnsiString;
  var
    LT: TSystemTime;
begin
  GetLocalTime(LT);
  with LT do Result := Format('[%04d:%02d:%02d::%02d:%02d:%02d] ', [ wYear, wMonth, wDay, wHour, wMinute, wSecond ]);
end;

procedure TMainForm.FormCreate(Sender: TObject);
var
  iCom, ComCt: Integer;
  hCom: THandle;
  Status: LongWord;

begin
  Randomize;

  ComCt := 0;
  for iCom := 1 to 255 do begin
    hCom := CreateComFile('\\.\COM' + IntToStr(iCom));
    Status := GetLastError;
    if hCom <> INVALID_HANDLE_VALUE then CloseHandle(hCom);
    if (Status <> ERROR_SUCCESS) and (Status <> ERROR_ACCESS_DENIED) then Continue;

    if ComCt = 0 then
      COMEditA.Text := IntToStr(iCom)
    else
      COMEditB.Text := IntToStr(iCom)
    ;
    Inc(ComCt);
    if ComCt = 2 then Break;
  end;

  case ComCt of
    0: COMLabel.Caption := 'No ports detected; please enter two.';
    1: COMLabel.Caption := 'One port detected; please enter another.';
    2: COMLabel.Caption := 'Click Start to start test.';
  end;

  ListPossibleBaudCodes;
end;

procedure TMainForm.COMEditChange(Sender: TObject);
var
  Dummy: LongWord;
  Code, ComCt: Integer;

begin
  ComCt := 0;
  Val(COMEditA.Text, Dummy, Code);
  if Code = 0 then Inc(ComCt);
  Val(COMEditB.Text, Dummy, Code);
  if Code = 0 then Inc(ComCt);

  case ComCt of
    0: COMLabel.Caption := 'No ports selected; please enter two.';
    1: COMLabel.Caption := 'One port selected; please enter another.';
    2: COMLabel.Caption := 'Click Start to start test.';
  end;
end;

procedure TMainForm.DisableControls;
begin
end;

procedure TMainForm.EnableControls;
begin
end;

procedure TMainForm.AMErrorStr(var Msg: TMessage);
begin
  if Msg.LParam <> LItsMe then Exit;

  ResultMemo.Lines.Add( PAnsiString(Msg.WParam)^);
  Inc(ErrCt);
  if (ErrCt >= 1024) and not bAborted then begin
    ResultMemo.Lines.Add('Too many errors, aborting...');
    PostMessage(Handle, AM_STOP, 0, 0);
    bAborted := True;
  end;
end;

procedure TMainForm.AMStop(var Msg: TMessage);
begin
  if Length(WorkerThread) = 0 then Exit;
end;

procedure TMainForm.AMByteCt(var Msg: TMessage);
begin
  if Msg.LParam <> LItsMe then Exit;

  COMLabel.Caption := IntToStr(Msg.WParam) + 'MB without error...';
end;

procedure TMainForm.lbPossibleBaudsClick(Sender: TObject);
begin
  with (Sender as tlistbox) do begin
    if ItemIndex = -1 then exit;
    ListPossibleBaudCodes(items[itemindex]);
  end;
end;

function TMainForm.IntToBaudCodeVerbose(v:Longword; verbose:Boolean):String;
  const
    labelStrings:array[false..true, 0..6] of string =
    (  {false=} ('',' ',' ',' ',' ',' ',' '),
        {true=} ('',' USR:',' SCR:',' DLH:',' DLL:',' M:',' N:')
    );
  var
    binary: String;
begin
  binary := IntToBin(v, 32);
  baudCodeUSR.Text := copy(binary,  1,  4);
  baudCodeSCR.Text := copy(binary,  5,  4);
  baudCodeDLH.Text := copy(binary,  9,  8);
  baudCodeDLL.Text := copy(binary, 17,  8);
  baudCodeM.Text   := copy(binary, 25,  5);
  baudCodeN.Text   := copy(binary, 30,  3);
  Result :=   labelStrings[verbose, 0]
            + copy(binary,  1,  1) // MSb
            + labelStrings[verbose, 1]
            + copy(binary,  2,  3) // any non-zero means "user defined baud code, instead of baud rate"
            + labelStrings[verbose, 2]
            + copy(binary,  5,  4) // SCR
            + labelStrings[verbose, 3]
            + copy(binary,  9,  8) // DLH
            + labelStrings[verbose, 4]
            + copy(binary, 17,  8) // DLL
            + labelStrings[verbose, 5]
            + copy(binary, 25,  5) // M
            + labelStrings[verbose, 6]
            + copy(binary, 30,  3) // N
           ;
end;

function IntToBaudCodeBinary(v:Longword):string;
begin
  MainForm.IntToBaudCodeVerbose(v, False);
end;

procedure TMainForm.lbWaysToGetBaudClick(Sender: TObject);
  var
    tokens: TStringList;
    Baud: TBaudInfo;
    msg, tempbinary:string;
    bw: Longword;
    tempvalue: Longword;

  function SampleClock: Longword;
  begin
    with Baud do Result := 16 - SCR;
  end;

  function Prescaler:longword;
  begin
    with Baud do Result := (1 shl M) * (SampleClock + N);
  end;

begin
  tokens := TStringList.Create;
  with (sender as TListBox) do begin
    if itemindex = -1 then exit;
    ;
    TokenizeCSV(copy(items[itemindex], pos('=', items[itemindex])+1, MAXINT), tokens, separator);

    if tokens.Count < 4 then Exit;

    with Baud do begin
      N := StrToInt(tokens[1]);
      M := StrToInt(tokens[2]);
      SCR := StrToInt(tokens[3]) and $F;
      DLL := StrToInt(tokens[4]);
      msg := copy(items[itemindex],0,pos('=',items[itemindex])) 
          + 'N=' + IntToStr(N) + separator
          + 'M=' + IntToStr(M) + separator
          + 'SCR=' + IntToStr(SCR) + separator
          + 'DLL=' + IntToStr(DLL) + separator
          + FloatToStrF(SlowClock / (DLL * Prescaler), ffFixed, 9, 0)
    ;

      Baud.Rate := FastClock / (DLL * Prescaler);
      BaudLabel.Caption := FloatToStr(1/1000000 * Round(Baud.Rate)) + #13#10'Megabaud'#13#10 + FloatToStr(1/2000000 * Round(Baud.Rate)) + ' Hz';
      Baud.Code :=    (N and $F)
                   or ((M and $1F) shl 3)
                   or ((DLL and $FFFF) shl 8)
                   or ((SCR and $F) shl 24)
                   or $70000000
      ;
      tempbinary := IntToBin(Baud.Code, 32);
      lbBaudCodeBinary.Text := 'Attempting to set Baud ' + IntToStr(trunc(Baud.Rate)) + #13#10
                                  + '0x' + IntToHex(Baud.Code, 8)
                                  + '='
                                  + tempbinary + #13#10
                                  + IntToBaudCodeVerbose(Baud.Code, FALSE) + #13#10
                                  + IntToBaudCodeVerbose(Baud.Code, TRUE) + #13#10

                               ;
      caption := msg + ' --> ' + IntToHex(Baud.Code, 8);

      ResultMemo.Lines.add(caption);

      if not cbTestOnSelectBaud.Checked then exit;

      DisableControls;
      ResultMemo.Lines.Clear;
      ErrCt := 0;
      bAborted := False;

      hComA := CreateComFile('\\.\COM' + COMEditA.Text);
      hComB := CreateComFile('\\.\COM' + COMEditB.Text);
      if (hComA = INVALID_HANDLE_VALUE) or (hComB = INVALID_HANDLE_VALUE) then begin
        if hComA <> INVALID_HANDLE_VALUE then CloseHandle(hComA);
        if hComB <> INVALID_HANDLE_VALUE then CloseHandle(hComB);
        if hComA = INVALID_HANDLE_VALUE then ResultMemo.Lines.Add('Failed to open COM' + COMEditA.Text + '.');
        if hComB = INVALID_HANDLE_VALUE then ResultMemo.Lines.Add('Failed to open COM' + COMEditB.Text + '.');
        EnableControls;
        Exit;
      end;

     { if Baud.Rate > 921600 then
        ConfigureCom(hComA, Baud.Code)
      else
      }
        ConfigureCom(hComA, trunc(Baud.Rate));
        ConfigureCom(hComA, Baud.Code);

      msg := StringOfChar('U',10);
      PersistentWriteFile(hComA, pchar(msg), length(msg), bw);
      EnableControls;

      CloseHandle(hComA);
      CloseHandle(hComB);

      ResultMemo.Lines.Add('Simple Test done.');
    end; // with baud
  end;// with sender
end;

procedure TMainForm.Button1Click(Sender: TObject);
  var
    msg:string;
    bw: Longword;
begin
      DisableControls;
      ResultMemo.Lines.Clear;
      ErrCt := 0;
      bAborted := False;

      hComA := CreateComFile('\\.\COM' + COMEditA.Text);
      if (hComA = INVALID_HANDLE_VALUE) then begin
        if hComA <> INVALID_HANDLE_VALUE then CloseHandle(hComA);
        if hComA = INVALID_HANDLE_VALUE then ResultMemo.Lines.Add('Failed to open COM' + COMEditA.Text + '.');
        EnableControls;
        Exit;
      end;

      ConfigureCom(hComA, (Sender as TButton).Tag);

      msg := StringOfChar('U',10);
      PersistentWriteFile(hComA, pchar(msg), length(msg), bw);
      EnableControls;

      CloseHandle(hComA);

      ResultMemo.Lines.Add('Simple Test done.');
end;

end.

